/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */


/*
 * functions common to all database tools.
 */

#include "server.cmdline.h"
#include "server.h"
#include "afs.h"
#include <dirent.h> /* readdir() */
#include <sys/stat.h> /* stat */
#include <sys/types.h> /* mode_t */

/*
 * return 1 if name matches any supported audio format
 */
static int match_audio_file_name(char *dir, char *name)
{
	int i, len = strlen(name);
	const char *pattern[] = {SUPPORTED_AUDIO_FORMATS_ARRAY};
        for (i = 0; pattern[i]; i++) {
                const char *p = pattern[i];
                int plen = strlen(p);
                if (len < plen + 1)
                        continue;
                if (name[len - plen - 1] != '.')
                        continue;
                if (strcasecmp(name + len - plen, p))
                        continue;
		return 1;
        }
        return 0;
}

/*
 * recusively traverse the current directory, search
 * for regular files
 */
int find_audio_files(int (*f)(const char *, const char *))
{
	DIR *dir;
	struct dirent *entry;
	char cwd[_POSIX_PATH_MAX + 1];
	struct stat s;
	int ret = -1;

	if (!getcwd(cwd, sizeof(cwd)))
		return -1;
	dir = opendir(".");
	if (!dir)
		return -1;
	/* scan cwd recursively */
	while ((entry = readdir(dir))) {
		mode_t m;
//		printf("checking %s/%s \n", cwd, entry->d_name);
		if (!strcmp(entry->d_name, "."))
			continue;
		if (!strcmp(entry->d_name, ".."))
			continue;
		if (lstat(entry->d_name, &s) == -1)
			goto out;
		m = s.st_mode;
		if (!S_ISREG(m) && !S_ISDIR(m)) /* skip links, sockets, ... */
			continue;
		if (S_ISREG(m)) { /* regular file */
			if (!match_audio_file_name(cwd, entry->d_name))
				continue;
			if (f(cwd, entry->d_name) < 0)
				goto out;
			continue;
		}
		/* directory */
		if (chdir(entry->d_name) == -1)
			goto out;
		find_audio_files(f);
		if (chdir(cwd) == -1)
			goto out;
	}
	ret = 1;
out:
	closedir(dir);
	return ret;
}
